extends RigidBody2D

export(int) var SPEED = 55
#export(int) var ARMOR = 9
#export(int) var COLLISION_DAMAGE = 1
export(int) var SCORE_POINTS = 10
export(int) var SPAWN_POWERUP_PERCENTAGE = 3
export(int) var SPAWN_FUEL_PERCENTAGE = 20
export(bool) var DESTROY_ON_SCREEN_EXIT = true

var is_destroyed = false
var is_invincible = false
signal enemy_exited
onready var CollisionSound = $CollisionSound
onready var Stats = $Stats

const ScoreEffect = preload("res://Effects/HitEffects/ScoreEffect.tscn")
const ExplosionEffect = preload("res://Effects/Explosions/ExplosionBig.tscn")
const HitEffect = preload("res://Effects/HitEffects/HitEffect2.tscn")
const BlackSunPowerUp = preload("res://Effects/PowerUps/BlackSun.tscn")
const Fuel = preload("res://Effects/PowerUps/Fuel.tscn")


func _ready():
	randomize()
	var main = get_tree().current_scene
	if main.is_in_group('stage'):
		connect("enemy_exited", main, "_on_BaseEnemy_exited")


func spawn_blacksun():
	var rand = randi() % 100
	if rand < SPAWN_POWERUP_PERCENTAGE:
		Utils.defer_instance_to_scene(BlackSunPowerUp, global_position)


func spawn_fuel():
	var rand = randi() % 100
	if rand < SPAWN_FUEL_PERCENTAGE:
		Utils.defer_instance_to_scene(Fuel, global_position)


func create_exposion_effect():
	Utils.defer_instance_to_scene(ExplosionEffect, global_position)


func create_hit_effect(at_position=null):
	if at_position == null:
		at_position = global_position
	Utils.defer_instance_to_scene(HitEffect, at_position)


func create_score_label(points: int):
	var score_label = Utils.add_instance_to_scene(ScoreEffect, global_position)
	score_label.message = "+{num}".format({'num': points})
	var current_scene = get_tree().current_scene
	if current_scene.is_in_group('stage'):
		current_scene.score += points


func destroy():
	is_destroyed = true
	create_exposion_effect()
	create_score_label(SCORE_POINTS)
	spawn_blacksun()
	spawn_fuel()
	queue_free()


func apply_damage(damage_points):
	if not is_invincible:
		Stats.health -= damage_points


func get_vector_to_player(from_position=global_position, normalized=true) -> Vector2:
	var vector: Vector2 = GlobalData.ship_position - from_position
	if normalized:
		vector = vector.normalized()
	return vector


func _on_BaseEnemy_area_entered(ship):
	destroy()
	ship.destroy()


func _on_VisibilityNotifier2D_screen_exited():
	if not is_destroyed:
		var main = get_tree().current_scene
		if main.is_in_group('stage'):
			emit_signal("enemy_exited")
	if DESTROY_ON_SCREEN_EXIT:
		queue_free()


func _on_BaseEnemy_body_entered(body):
	# Two enemies collided into each other
	if body.is_in_group('enemies'):
		CollisionSound.play()
		create_hit_effect((global_position + body.global_position) / 2 )
#		apply_damage(body.COLLISION_DAMAGE)


func _on_Stats_died():
	destroy()


func _on_Hurtbox_hit(damage):
	apply_damage(damage)
