extends "res://Enemy/BaseEnemy.gd"

export(Vector2) var SPEED_FOLLOW = Vector2(65, 35)
export(int) var SPEED_SLOW = 225
export(int) var SPEED_FAST = 275
export(int) var SPEED_SHOOT = 30
var speed = SPEED_SLOW

enum states {INTRO, AVOID, JUMP, ATTACK}
enum attack_states {AVOID, SHOOT, JUMP, RUSH}
var state = states.INTRO
var attack_state = attack_states.AVOID

var motion: Vector2 = Vector2.ZERO
var last_position: Vector2 = Vector2.ZERO
var last_animation
var warned_about_shekels = false

const Shekel = preload("res://Effects/KikeItems/Dollar.tscn")
const Bullet = preload("res://Effects/Weapons/KikeStarBullet.tscn")
const PowerUpSSGun = preload("res://Effects/PowerUps/PowerUpSSGun.tscn")

onready var ShootTimer = $ShootTimer
onready var PunchTimer = $PunchTimer
onready var HappeningTimer = $HappeningTimer
onready var Muzzle = $Sprite/Body/Head/Muzzle
onready var HeadPosition = $Sprite/Body/Head/HeadPosition
onready var AnimationPlayer = $AnimationPlayer
onready var MovementTween = $MovementTween

signal shekels
signal kike_died

func _ready():
	is_invincible = true
	Music.switch_to_kike()
	var main = get_tree().current_scene
	if main.is_in_group('stage'):
		connect("kike_died", main, "_on_Kike_died")
	#AnimationPlayer.play("Intro")

func destroy():
	Music.switch_to_music()
	emit_signal("kike_died")
	Utils.defer_instance_to_scene(PowerUpSSGun, Muzzle.global_position)
	.destroy()


func take_aim(auto_fire=false):
	if ShootTimer.is_stopped():
		var aiming_position = Muzzle.global_position
		var aiming_direction = Vector2.RIGHT * scale.x
		var vector_to_player = get_vector_to_player(aiming_position, false)
		#var distance_to_player = vector_to_player.length()
		var angle_to_player = aiming_direction.angle_to(vector_to_player)
		
		
		if auto_fire or angle_to_player < TAU * .1:
#			if vector_to_player.x < 0:
#				print("Player is LEFT")
#			else:
#				print("Player is RIGHT")
#			print("Distance: %f" % distance_to_player)
#			print("Angle: %f" % angle_to_player)
#			print("Vector to Player: %s" % vector_to_player)
			
			aiming_direction = aiming_direction.rotated(rand_range(-.2, .2))
			var bullet = Utils.defer_instance_to_scene(Bullet, Muzzle.global_position)
			bullet.apply_central_impulse(aiming_direction * 200)
			AnimationPlayer.play("Shoot")
			
			ShootTimer.start()


func take_aim2(auto_fire=false):
	if ShootTimer.is_stopped():
		var aiming_position = Muzzle.global_position
		var aiming_direction = Vector2.RIGHT * scale.x
		var vector_to_player = get_vector_to_player(aiming_position, false)
		var aiming_factor = aiming_direction.dot(vector_to_player.normalized()) * -1
		var distance_factor = vector_to_player.length() / 320
		
		aiming_factor = clamp(aiming_factor - 0.75, 0, 1) * 4
		distance_factor = 1 - clamp(distance_factor * 1.15, 0, 1)
		if auto_fire or (aiming_factor + distance_factor) >= 1.15:
			var shoot_angle = vector_to_player.angle() * 2
			aiming_direction = aiming_direction.rotated(rand_range(-shoot_angle, shoot_angle))
			var bullet = Utils.defer_instance_to_scene(Bullet, Muzzle.global_position)
			bullet.apply_central_impulse(aiming_direction * 200)
			AnimationPlayer.play("Shoot")
			ShootTimer.start()

func punch():
	if PunchTimer.is_stopped():
		AnimationPlayer.play("Punch")
		PunchTimer.start()

func throw_shekels():
	assert(HappeningTimer.is_stopped())
	var origin: Vector2 = global_position + Vector2(0, -25)
	var dir_vector: Vector2
	var num_shekels: int = 16
	var direction_range: float = TAU / 5
	var shekel
	
	for _i in range(num_shekels):
		dir_vector = Vector2.RIGHT * scale
		dir_vector = dir_vector.rotated(rand_range(-direction_range, direction_range))
		shekel = Utils.add_instance_to_scene(Shekel, origin + dir_vector * 50)
		shekel.apply_central_impulse(dir_vector * rand_range(.75, 1.25) * shekel.SPEED)
#	print("Happening Timer Start in throw_shekels")
	HappeningTimer.start()
	emit_signal("shekels")
	if not warned_about_shekels:
		VoiceBox.warn_shekels()
		warned_about_shekels = true

func apply_damage(damage_points):
	.apply_damage(damage_points)
	if randi() % 30 == 0:
		VoiceBox.insult_kikes()

func _physics_process(delta):
	motion = global_position - last_position
	
	if not MovementTween.is_active():
		apply_state(delta)
		scale.x = sign(get_vector_to_player(global_position, false).x)
	else: 
		apply_state_during_tween_movement(delta)
	last_position = global_position


# ---

func get_state_s():
	match state:
		states.AVOID:
			return "AVOID"
		states.INTRO:
			return "INTRO"
		states.JUMP:
			return "JUMP"
		states.ATTACK:
			var ret = "ATTACK/"
			match attack_state:
				attack_states.AVOID:
					return ret + "AVOID"
				attack_states.JUMP:
					return ret + "JUMP"
				attack_states.RUSH:
					return ret + "RUSH"
				attack_states.SHOOT:
					return ret + "SHOOT"


func _on_MovementTween_tween_all_completed():
#	print("tween completed")
	assert(HappeningTimer.is_stopped())
#	print("Current state: %s" % get_state_s())
	
	match state:
		states.INTRO:
			pass
		states.AVOID:
#			print("Happening Timer in MovementTweenCompleted AVOID")
			HappeningTimer.start()
		states.JUMP:
			set_state(states.AVOID)
#			print("Happening Timer in MovementTweenCompleted JUMP")
			HappeningTimer.start()
		states.ATTACK:
			match attack_state:
				attack_states.AVOID:
					AnimationPlayer.stop()
					AnimationPlayer.current_animation = "Angry"
				attack_states.JUMP:
					set_attack_state(attack_states.AVOID)
#					print("Happening Timer in MovementTweenCompleted attack/avoid")
					HappeningTimer.start()
				attack_states.SHOOT:
					set_attack_state(attack_states.AVOID)
					HappeningTimer.start()
				attack_states.RUSH:
					set_attack_state(attack_states.AVOID)
					HappeningTimer.start()
	
	


# ---


func apply_state(delta):
	assert(not MovementTween.is_active())
	match state:
		states.INTRO:
			pass
		states.AVOID:
			avoid_player()
			#take_aim()
		states.JUMP:
			take_aim()
		states.ATTACK:
			apply_attack_state(delta)


func apply_attack_state(_delta):
	assert(not MovementTween.is_active())
	match attack_state:
		attack_states.AVOID:
			attack_avoid()
		attack_states.JUMP:
			take_aim()
		attack_states.RUSH:
			pass
		attack_states.SHOOT:
			take_aim()


func apply_state_during_tween_movement(_delta):
	assert(MovementTween.is_active())
	match state:
		states.INTRO:
			pass
		states.AVOID:
			pass
		states.JUMP:
			take_aim()
		states.ATTACK:
			match attack_state:
				attack_states.AVOID:
					pass
				attack_states.JUMP:
					take_aim()
				attack_states.RUSH:
					pass
				attack_states.SHOOT:
					take_aim(true)


func set_state(new_state):
	if state != new_state:
		state = new_state
		AnimationPlayer.stop()
		ShootTimer.stop()
		HappeningTimer.stop()
		PunchTimer.stop()
		match state:
			states.AVOID:
#				print("Set state to states.AVOID")
				AnimationPlayer.play("RubHands")
				HappeningTimer.wait_time = .5
				ShootTimer.wait_time = 0.4
				speed = SPEED_SLOW
				# print("Happening Timer in set_state")
				# HappeningTimer.start()
			states.JUMP:
#				print("Set state to states.JUMP")
				AnimationPlayer.play("Running")
				speed = SPEED_SLOW
			states.ATTACK:
#				print("Set state to states.ATTACK")
				ShootTimer.wait_time = 0.2
				HappeningTimer.wait_time = 0.75
				speed = SPEED_SLOW
				set_attack_state(attack_states.AVOID)
				jump_to_safe_corner()


func set_attack_state(new_state):
	if attack_state != new_state:
		attack_state = new_state
		AnimationPlayer.stop()
		ShootTimer.stop()
		HappeningTimer.stop()
		PunchTimer.stop()
		match attack_state:
			attack_states.AVOID:
#				print("Set attack_state to attack_states.AVOID")
				speed = SPEED_SLOW
				AnimationPlayer.play("Angry")
				# print("Happening Timer in set_attack_state")
				# HappeningTimer.start()
			attack_states.SHOOT:
#				print("Set attack_state to attack_states.SHOOT")
				speed = SPEED_SHOOT
				AnimationPlayer.play("Shoot")
			attack_states.JUMP:
#				print("Set attack_state to attack_states.JUMP")
				speed = SPEED_SLOW
				AnimationPlayer.play("Running")
			attack_states.RUSH:
#				print("Set attack_state to attack_states.RUSH")
				speed = SPEED_SLOW
				AnimationPlayer.play("Mad")


func jump_to_safe_corner():
	var top_left: Vector2 = Vector2(40, 70)
	var bottom_left: Vector2 = Vector2(40, 180)
	var top_right: Vector2 = Vector2(280, 70)
	var bottom_right: Vector2 = Vector2(280, 180)
	var goto_position: Vector2 = Vector2.ZERO
	
	ShootTimer.stop()
	HappeningTimer.stop()
	PunchTimer.stop()
	
	if global_position.x < GlobalData.ship_position.x:
		if GlobalData.ship_position.y < 90:
			goto_position = bottom_left
		else:
			goto_position = top_left
	else:
		if GlobalData.ship_position.y < 90:
			goto_position = bottom_right
		else:
			goto_position = top_right
	var duration = global_position.distance_to(goto_position) / speed
	MovementTween.interpolate_property(self,
		"global_position",
		global_position,
		goto_position,
		duration,
		MovementTween.TRANS_CUBIC,
		MovementTween.EASE_OUT)
	MovementTween.start()


func switch_corner():
#	print("Switching Corners")
	var is_left: bool = global_position.x < 160
	var is_top: bool = global_position.y < 90
	var top_left: Vector2 = Vector2(40, 70)
	var bottom_left: Vector2 = Vector2(40, 180)
	var top_right: Vector2 = Vector2(280, 70)
	var bottom_right: Vector2 = Vector2(280, 180)
	var goto_position: Vector2
	
	assert(not MovementTween.is_active())
	
	HappeningTimer.stop()
	ShootTimer.stop()
	PunchTimer.stop()
	
	if is_left:
		if is_top:
			goto_position = bottom_left
		else:
			goto_position = top_left
	else:
		if is_top:
			goto_position = bottom_right
		else:
			goto_position = top_right
	
	var duration = global_position.distance_to(goto_position) / speed
	MovementTween.interpolate_property(self,
		"global_position",
		global_position,
		goto_position,
		duration,
		Tween.TRANS_LINEAR,
		Tween.EASE_OUT)
	MovementTween.start()


func switch_sides():
#	print("Switching Sides")
	var is_left: bool = global_position.x < 160
	var is_top: bool = global_position.y < 90
	var top_left: Vector2 = Vector2(40, 70)
	var bottom_left: Vector2 = Vector2(40, 180)
	var top_right: Vector2 = Vector2(280, 70)
	var bottom_right: Vector2 = Vector2(280, 180)
	var goto_position: Vector2
	
	assert(not MovementTween.is_active())
		
	HappeningTimer.stop()
	ShootTimer.stop()
	PunchTimer.stop()
	
	if is_left:
		if is_top:
			goto_position = bottom_right
		else:
			goto_position = top_right
	else:
		if is_top:
			goto_position = bottom_left
		else:
			goto_position = top_left
	
	var duration = global_position.distance_to(goto_position) / speed
	MovementTween.interpolate_property(self,
		"global_position",
		global_position,
		goto_position,
		duration,
		Tween.TRANS_LINEAR,
		Tween.EASE_OUT)
	MovementTween.start()


func attack_avoid():
	"is called when no movement happens, and state is attack_states.AVOID"
	assert(MovementTween.is_active() == false)
	
	if abs(get_vector_to_player(HeadPosition.global_position, false).y) < 50:
		set_attack_state(attack_states.JUMP)
		switch_corner()


func avoid_player():
	assert(MovementTween.is_active() == false)
	
	if abs(get_vector_to_player(HeadPosition.global_position, false).y) < 50:
		set_state(states.JUMP)
		switch_corner()


func follow_player(delta):
	motion = -get_vector_to_player()
	motion *= speed
	position += motion * delta


func attack_player():
	var origin_to_head_y = global_position.y - HeadPosition.global_position.y
	var goto_position = Vector2(global_position.x, GlobalData.get_ship_position().y + origin_to_head_y)
	
	var duration = global_position.distance_to(goto_position) / speed
#	print("Attack Speed: %d" % speed)
#	print("Attack state: %s" % get_state_s())
	
	HappeningTimer.stop()
	ShootTimer.stop()
	PunchTimer.stop()
	
	MovementTween.interpolate_property(self, 
		"global_position", 
		global_position, 
		goto_position, 
		duration, 
		Tween.TRANS_ELASTIC, 
		Tween.EASE_OUT)
#	print("Starting attack tween")
	MovementTween.start()


func _on_HappeningTimer_timeout():
	match state:
		states.INTRO:
			pass
		states.AVOID:
			AnimationPlayer.play("GiveShekels")
		states.JUMP:
			pass
		states.ATTACK:
			if randi() % 2:
#				print("Attacking Player")
				set_attack_state(attack_states.SHOOT)
				attack_player()
				return
			else:
#				print("switching sides")
				set_attack_state(attack_states.RUSH)
				switch_sides()
				return


func _on_AnimationPlayer_animation_finished(_anim_name):
	match state:
		states.INTRO:
			set_state(states.AVOID)
			is_invincible = false
			AnimationPlayer.current_animation = "RubHands"
		states.AVOID:
			AnimationPlayer.current_animation = "RubHands"
		states.JUMP:
			AnimationPlayer.current_animation = "Running"
		states.ATTACK:
			match attack_state:
				attack_states.AVOID:
					AnimationPlayer.current_animation = "Angry"
				attack_states.JUMP:
					AnimationPlayer.current_animation = "Running"
				attack_states.SHOOT:
					AnimationPlayer.current_animation = "Shoot"
				attack_states.RUSH:
					AnimationPlayer.current_animation = "Mad"
	AnimationPlayer.play()


func _on_Stats_changed():
	if not is_invincible:
		AnimationPlayer.play("Hit")
	if not HappeningTimer.is_stopped():
		HappeningTimer.stop()
#		print("Starting Happening Timer Restart in stats_changed")
		HappeningTimer.start()
	if Stats.health < Stats.MAX_HEALTH / 2:
		set_state(states.ATTACK)
	else:
		set_state(states.AVOID)


func _on_PunchArea_area_entered(_area):
	if PunchTimer.is_stopped():
		punch()
