extends Area2D

const Bullet = preload("res://Effects/Weapons/SwastikaBullet.tscn")
const SSBullet = preload("res://Effects/Weapons/SSBullet.tscn")
const BlackSun = preload("res://Effects/Weapons/BlackSun.tscn")
const ExplosionEffect = preload("res://Effects/Explosions/ExplosionEffect.tscn")
const audio_pling = preload("res://Resources/Sound/Ping/ping 02.wav")
const audio_crash = preload("res://Resources/Sound/Collision/collision 2.wav")
const Message = preload("res://Effects/Messages/BaseMessage.tscn")

var rng = RandomNumberGenerator.new()

export(int) var SPEED = 100
export(int) var SCATTER = 50
export(int) var num_blacksun_missiles = 1
export(bool) var has_ss_gun = false
const exhaust_base_speed = 150

var screensize
var movement = Vector2.ZERO
export(bool) var is_invincible = false

onready var fire_cooldown = $CooldownTimer
onready var blacksun_cooldown = $BlacksunTimer
onready var ss_cooldown = $SSTimer
onready var exhaust_particles = $ExhaustParticles
onready var animation_player = $AnimationPlayer
onready var stats = $Stats

signal ship_destroyed
signal powerup
signal fuelup(amount)
signal damage


func print_type():
	print("Type Ship")


func display_message(message: String):
	var main = get_tree().current_scene
	var obj = Message.instance()
	obj.text = message
	main.call_deferred("add_child", obj)


func _ready():
	screensize = get_viewport_rect().size
	rng.randomize()
	exhaust_particles.initial_velocity = exhaust_base_speed


func _process(delta):
	movement.x = int(Input.is_action_pressed("ui_right")) - int(Input.is_action_pressed("ui_left"))
	movement.y = int(Input.is_action_pressed("ui_down")) - int(Input.is_action_pressed("ui_up"))

	if Input.is_action_pressed("ui_fire_swastika"):
		if fire_cooldown.is_stopped():
			fire_cooldown.start()
			fire_bullet()
	if Input.is_action_pressed("ui_fire_blacksun"):
		if blacksun_cooldown.is_stopped():
			blacksun_cooldown.start()
			fire_blacksun()
	if Input.is_action_pressed("ui_fire_ss") and has_ss_gun:
		if ss_cooldown.is_stopped():
			ss_cooldown.start()
			fire_ss()
			
	if movement != Vector2.ZERO:
		position += movement.normalized() * SPEED * delta
		position.x = clamp(position.x, 8, screensize.x - 8)  # 8 because sprite is 16 px wide
		position.y = clamp(position.y, 8, screensize.y - 8)
#		GlobalData.ship_position = position
	

func fire_bullet():
	var randint = rng.randi_range(-SCATTER, SCATTER)
	var bullet = Utils.defer_instance_to_scene(Bullet, global_position)
	bullet.linear_velocity.y = randint


func fire_blacksun():
	if num_blacksun_missiles > 0:
		Utils.defer_instance_to_scene(BlackSun, global_position)
		num_blacksun_missiles -= 1
		emit_signal("powerup")


func fire_ss():
	var mouse_pos = get_local_mouse_position().normalized()
	var angle_to_mouse = get_local_mouse_position().angle() # + TAU / 4
	var bullet = Utils.defer_instance_to_scene(SSBullet, global_position)
	bullet.rotation = angle_to_mouse
	bullet.apply_central_impulse(mouse_pos * bullet.speed)



func destroy():
#	print("Ship.destroy() start")
	Utils.defer_instance_to_scene(ExplosionEffect, global_position)
	GlobalData.is_counting = false
	VoiceBox.clear_playlist()
	emit_signal("ship_destroyed")
	queue_free()
#	print("Ship.destroy() end")


func power_up(powerup_type, add_val):
	var audioplayer = $AudioStreamPlayer
	audioplayer.stream = audio_pling
	audioplayer.play()
	
	if powerup_type == "blacksun":
		num_blacksun_missiles += add_val
	elif powerup_type == "ss-gun":
		has_ss_gun = true
		VoiceBox.play_ss_gun()
		display_message("You found the SS-Gun")
	emit_signal("powerup")


func fuel_up(amount):
	var audioplayer = $AudioStreamPlayer
	audioplayer.stream = audio_pling
	audioplayer.play()
	emit_signal("fuelup", amount)


func _on_Hurtbox_hit(damage):
	if not is_invincible:
		stats.health -= damage 
		emit_signal("damage")
		var audioplayer = $AudioStreamPlayer
		audioplayer.stream = audio_crash
		audioplayer.play()
		if stats.health > 0:
			var damage_ratio: float = float(stats.health) / stats.MAX_HEALTH
			if damage_ratio <= 0.4:
				VoiceBox.warn_almost_dead()
				
			is_invincible = true
			animation_player.play("Invincible")
			yield(get_tree().create_timer(2.0), "timeout")
			is_invincible = false


func _on_Stats_died():
	destroy()
