extends Node

var score setget set_score, get_score

onready var score_label = $ScoreLabel
onready var blacksun_label = $blacksun_small/BlacksunLabel
onready var fuelbar = $FuelBar
onready var armorbar = $ArmorBar
onready var ship = $Ship
onready var LevelTimer = $LevelTimer
onready var EnemySpawner = $EnemySpawner

const AntifaSpawner = preload("res://Enemy/AntifaSpawner.tscn")
const KikeBoss = preload("res://Enemy/KikeBoss.tscn")

const Message = preload("res://Effects/Messages/BaseMessage.tscn")

func _ready():
	GlobalData.is_counting = true
	GlobalData.score = 0
	fuelbar.FUEL = fuelbar.FUEL
	if GlobalData.first_round:
		VoiceBox.play_welcome()
		yield(get_tree().create_timer(5), "timeout")
	LevelTimer.start()
	EnemySpawner.start()


func display_message(message: String):
	var main = get_tree().current_scene
	var obj = Message.instance()
	obj.text = message
	main.call_deferred("add_child", obj)


func set_score(value):
	var old_score = GlobalData.score
	GlobalData.score = value
	score_label.text = "Score: {score}".format({'score': GlobalData.score})
	if old_score < 1488 and value > 1488:
		display_message("1488 points")
		VoiceBox.play_1488()


func get_score():
	return GlobalData.score


func _on_Ship_ship_destroyed():
#	print("Changing to GameOverScene")
	yield(get_tree().create_timer(2), "timeout")
	get_tree().change_scene("res://GameOverScreen.tscn")
#	print("Changed to GameOverScene")


func _on_Ship_powerup():
	blacksun_label.text = "{num}".format({'num': ship.num_blacksun_missiles})


func _on_Ship_fuelup(amount):
	fuelbar.FUEL += amount


func _on_FuelBar_fuel_30pct():
	display_message("kill more")
	VoiceBox.play_kill_more()


func _on_FuelBar_fuel_20pct():
	display_message("kill more npcs")
	VoiceBox.play_kill_more_npcs()


func _on_FuelBar_fuel_10pct():
	display_message("fuel low")
	VoiceBox.play_fuel_low()


func _on_FuelBar_fuel_empty():
	#print("fuelbar empty - calling ship.destroy()")
	if ship:
		ship.destroy()


func _on_BaseEnemy_exited():
	fuelbar.FUEL -= 1


func _on_Ship_damage():
	armorbar.update_health()


func _on_Dollar_exited():
	#print("Dollar exited")
	EnemySpawner.start_spawning()
	EnemySpawner.increase_speed(0.99)


func _on_Kike_died():
	EnemySpawner.start_spawning()
	display_message("Mission Accomplished!")
	VoiceBox.play_mission_accomplished()


func switch_to_antifa_spawner():
	EnemySpawner = Utils.defer_instance_to_scene(AntifaSpawner, Vector2.ZERO)
	Utils.defer_instance_to_scene(KikeBoss, Vector2(250, 100))
	display_message("Gas the Kikes!\nRacewar Now!")
	VoiceBox.play_gas_the_kikes()


func _on_LevelTimer_timeout():
	#print("LevelTimer Timeout")
	EnemySpawner.queue_free()
	yield(get_tree().create_timer(5), "timeout")
	if self:
		switch_to_antifa_spawner()
